# Packages
library("prettyR")
library("lubridate")
library("dfexplore")
library("pander")
library("gmodels")
library("ggplot2")

source("config.R")

# Functions

# table with NA
my_table <- function(..., useNA = "ifany"){
    t <- table(..., useNA = useNA)
    row.names(t)[is.na(names(t))] <- "NA"
    if(length(dim(t)) == 2)
        colnames(t)[is.na(colnames(t))] <- "NA"
    return(t)
}

# plot with jittered points
jitter_plot <- function(x, y, ...){
    return(plot(jitter(x),
                jitter(y),
                ...))
}
# barplot with inclined x labels
my_barplot <- function(tab, ...){
    op <- par(mar = c(7.5,4,4,2)+.1)
    bp <- barplot(tab,
                  names.arg = "",
                  ...)
    text(bp, par("usr")[3] - par("usr")[4] * 0.02, labels = names(tab), srt = 45, adj = 1, xpd = T, cex = .9)
    par(op)
    return(bp)
}
# return TRUE if selected variables are 0 or NA
var_is_na_or_zero <- function(types_couv){
    return(apply(apply(data_ht[, paste("TYPE_COUV", types_couv, sep = "_")],
                       2,
                       function(x) {as.numeric(as.character(x))}),
                 1,
                 sum, na.rm = TRUE) == 0)
}

# loading data
col_classes <- c(IEP_IPP = "character")
data_ht <- read.csv2("../donnees/donnees.csv",
                     na.strings = c("", NA),
                     colClasses = col_classes)

# extraction of IEP / IPP
# patients_wrong_IEP <- data_ht[which(nchar(data_ht$IEP_IPP) != 9), "NUM_PATIENT"]
# IEP <- data_ht$IEP_IPP[grepl("^2.*", data_ht$IEP_IPP) & nchar(data_ht$IEP_IPP) == 9]
# IPP <- data_ht$IEP_IPP[which(! data_ht$IEP_IPP %in% IEP & nchar(data_ht$IEP_IPP) == 9)]

# Labels of levels
levels(data_ht$SEXE) <- c("femme", "homme")

# time variables
data_ht$DDN <- as.Date(data_ht$DDN, format = "%d/%m/%Y")
data_ht$DATE <- as.Date(data_ht$DATE, format="%d/%m/%y")
for (v in c("INDUCTION", "REVEIL", "DEBUT_SSPI", "DEBUT_ITV")){
    data_ht[, paste0("TPS_", v)] <- as.POSIXct(paste(data_ht$DATE,
                                                     data_ht[, paste0("HEURE_", v)]),
                                               format = "%Y-%m-%d %H:%M")
}

# if end of anaesthesia or start of surgery anterior to start of anaesthesia, adding one day
for (v in c("REVEIL", "DEBUT_SSPI", "DEBUT_ITV")){
    print(paste0("Heure ", v, " < heure de l'induction"))
    print(data_ht[which(difftime(data_ht[, paste0("TPS_", v)], data_ht$TPS_INDUCTION) < 0), c("NUM_PATIENT", "TPS_INDUCTION", paste0("TPS_", v))])
    
    data_ht[which(difftime(data_ht[, paste0("TPS_", v)], data_ht$TPS_INDUCTION) < 0), paste0("TPS_", v)] <- data_ht[which(difftime(data_ht[, paste0("TPS_", v)], data_ht$TPS_INDUCTION) < 0), paste0("TPS_", v)] + days(1)
}

data_ht[, paste0("HEURE_", c("INDUCTION", "REVEIL", "DEBUT_SSPI", "DEBUT_ITV"))] <- NULL

# computed variables
data_ht$AGE <- floor(time_length(difftime(data_ht$DATE, data_ht$DDN),
                                 unit = "years"))
data_ht$BMI <- data_ht$POIDS / (data_ht$TAILLE/100)^2
data_ht$DUREE_ANESTH <- as.numeric(difftime(data_ht$TPS_REVEIL, data_ht$TPS_INDUCTION, units = "min"))
# if duration of anaesthesia is missing, attempting to compute from arrival in recovery room (- 5 minutes)
data_ht$DUREE_ANESTH[is.na(data_ht$DUREE_ANESTH)] <- as.numeric(
    difftime(data_ht$TPS_DEBUT_SSPI[is.na(data_ht$DUREE_ANESTH)],
             data_ht$TPS_INDUCTION[is.na(data_ht$DUREE_ANESTH)],
             units = "min")) - 5
data_ht$DELAI_INDUCTION_CHIR <- as.numeric(difftime(data_ht$TPS_DEBUT_ITV, data_ht$TPS_INDUCTION, units = "min"))
# available data to determine "end of anesthesia" : awekening or arrival in recovery room
data_ht$TPS_FIN_DISPO <- as.factor(ifelse(!is.na(data_ht$TPS_REVEIL), "réveil",
                                          ifelse(!is.na(data_ht$TPS_DEBUT_SSPI), "SSPI", "aucun")))
data_ht$DUREE_CHIRURGIE <- data_ht$DUREE_ANESTH - data_ht$DELAI_INDUCTION_CHIR

# binary variables
for (v in names(data_ht)){
    if (class(data_ht[,v]) == "integer" & length(unique(na.omit(data_ht[,v]))) %in% c(1,2))
        data_ht[,v] <- factor(data_ht[,v])
}
for (v in c("ASA", "NUM_SALLE", "NUM_PATIENT")){
    data_ht[,v] <- factor(data_ht[,v])
}

# type of heating : suppress NA values (= 0)
type_couv <- c("TYPE_COUV_HC", "TYPE_COUV_BC", "TYPE_COUV_CC", "TYPE_COUV_MAT")
names(type_couv) <- c("haut corps", "bas corps", "corps complet", "matelas")
for (v in type_couv){
    data_ht[which(is.na(data_ht[,v])), v] <- 0
}

# type of surgery
surgery_types <- read.csv2("../donnees/types_chirurgies.csv",
                           stringsAsFactors = FALSE)
data_ht$INTERVENTION <- as.character(data_ht$INTERVENTION)
data_ht <- merge(data_ht,
                 surgery_types[,c("INTERVENTION", "TYPE_CHIRURGIE")],
                 by = "INTERVENTION",
                 all.x = TRUE)
data_ht$TYPE_CHIRURGIE <- factor(data_ht$TYPE_CHIRURGIE)

# removing extreme values
# - of axillary temperature (< 34 or > 38.5°C)
data_ht$TEMP_SSPI_AX_orig <- data_ht$TEMP_SSPI_AX
is.na(data_ht$TEMP_SSPI_AX[which(data_ht$TEMP_SSPI_AX > 38.5 | data_ht$TEMP_SSPI_AX < 34)]) <- TRUE
# - of room temperature (< 18)
data_ht$TEMP_SALLE_orig <- data_ht$TEMP_SALLE
is.na(data_ht$TEMP_SALLE[which(data_ht$TEMP_SALLE < 18)]) <- TRUE
# - of duration of anesthesia
data_ht$DUREE_ANESTH_orig <- data_ht$DUREE_ANESTH
is.na(data_ht$DUREE_ANESTH[which(data_ht$DUREE_ANESTH > 200)]) <- TRUE
# - of duration between induction and surgery
data_ht$DELAI_INDUCTION_CHIR_orig <- data_ht$DELAI_INDUCTION_CHIR
is.na(data_ht$DELAI_INDUCTION_CHIR[which(data_ht$DELAI_INDUCTION_CHIR > 80)]) <- TRUE
# - of duration of surgery
data_ht$DUREE_CHIRURGIE_orig <- data_ht$DUREE_CHIRURGIE
is.na(data_ht$DUREE_CHIRURGIE[which(data_ht$DUREE_CHIRURGIE > 180)]) <- TRUE

# contrast for regressions
contrasts(data_ht$TYPE_ANESTHESIE) <- contr.sum
mods <- row.names(contrasts(data_ht$TYPE_ANESTHESIE))
colnames(contrasts(data_ht$TYPE_ANESTHESIE)) <- mods[seq(length(mods) - 1)]

contrasts(data_ht$TYPE_CHIRURGIE) <- contr.sum
mods <- row.names(contrasts(data_ht$TYPE_CHIRURGIE))
colnames(contrasts(data_ht$TYPE_CHIRURGIE)) <- mods[seq(length(mods) - 1)]
rm(mods)
